package xamora.gp_dbc.config;

import cpw.mods.fml.client.event.ConfigChangedEvent;
import cpw.mods.fml.common.eventhandler.SubscribeEvent;

import xamora.gp_dbc.Main;
import xamora.gp_dbc.utils.Pos;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.ChunkCoordinates;
import net.minecraft.world.WorldServer;
import net.minecraftforge.common.config.Configuration;
import net.minecraftforge.common.config.Property;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

public class ConfigManager {

    private static Configuration config;
    public static boolean updateCheck = true;
    public static String configDir;

    public static final String CATEGORY_GP = "Galactic Patrol";

    public static String DIR_SAVE = "./" + Main.MODID + "/";

    public static byte MIN_ALIGN_TO_JOIN = 33;
    public static byte MAX_ALIGN_TO_JOIN = 100;

    public static int FORMAT_TIME_JAIL = 4;
    public static int DEFAULT_PLACE_CELL = 2;

    public static int TIME_HANDCUFFING = 5; // seconds
    public static int TIME_UNHANDCUFFING = 10; // seconds
    public static int TIME_TO_GO_IN_JAIL = 20; // seconds

    public static boolean OP_JAIL_MANAGER = true;

    static public int SEC_TO_BE_IN_JAIL = 120; // second
    static public String POS_EXIT_JAIL = ""; //"-247 66 275 20";
    static public Pos pos_exit_jail;

    static public double MULT_JAIL_TIME = 7.2;
    static public int REDUCE_ALIGN_KILL = 33;

    // 7.2 * 24000 = 172 800 = 48h
    static public int MAX_RATE_CRIMINAL = 24000;
    static public int MIN_RATE_TO_HAVE_POSITION = 1000;
    static public long MIN_TIME_BETWEEN_EACH_POSITION = 60; // 1 min
    static public long MAX_TIME_BETWEEN_EACH_POSITION = 2 * 60 * 60; // 2 hours

    static public boolean REINCARNATION_RESET_CRIMINAL = true;
    static public boolean REINCARNATION_RESET_GP = true;

    static public boolean NICKNAME_FOR_CRIMINAL = false;

    static public boolean CRIMINAL_QUIT_IN_FIGHT_GO_TO_JAIL = true;
    static public long CRIMINAL_TIME_TO_QUIT_IN_FIGHT = 20;

    public static void init(String configDir) {
        if (config != null)
            return ;

        ConfigManager.configDir = configDir;
        File path = new File(configDir + "/" + Main.MODID + ".cfg");

        config = new Configuration(path);
        loadConfiguration();
    }

    private static void loadConfiguration() {
        updateCheck = config.getBoolean("Check For Updates", Configuration.CATEGORY_GENERAL, true, "Allow this mod to check for updates.");

        List<String> order = new ArrayList<String>();

        DIR_SAVE = addVariableConfig(order, CATEGORY_GP, "DIR_SAVE",
                "Directory where data is saved. [default: " + "./" + Main.MODID + "/" + " ]", "./" + Main.MODID + "/").getString();

        MIN_ALIGN_TO_JOIN = (byte)addVariableConfig(order, CATEGORY_GP, "MIN_ALIGN_TO_JOIN",
                "Minimum alignment to join galactic patrol. [default: 33]", "33").getInt();

        MAX_ALIGN_TO_JOIN = (byte)addVariableConfig(order, CATEGORY_GP, "MAX_ALIGN_TO_JOIN",
                "Maximum alignment to join galactic patrol. [default: 100]", "100").getInt();

        POS_EXIT_JAIL = addVariableConfig(order, CATEGORY_GP, "POS_EXIT_JAIL",
                "Spawn point when a jailer leaves the prison. It's mandatory to put a player in jail. \"<x> <y> <z> <dimension id>\" [default: \"\"]", "").getString();
        try {
            String[] split = ConfigManager.POS_EXIT_JAIL.split(" ");
            pos_exit_jail = new Pos(Integer.parseInt(split[0]), Integer.parseInt(split[1]), Integer.parseInt(split[2]), Integer.parseInt(split[3]));
        } catch (Exception e) {
            System.out.println(e.getMessage());
            pos_exit_jail = null;
        }

        TIME_HANDCUFFING = (byte)addVariableConfig(order, CATEGORY_GP, "TIME_HANDCUFFING",
                "Time to handcuffing someone who is not K.O, in seconds. [default: 5]", "5").getInt();

        TIME_UNHANDCUFFING = (byte)addVariableConfig(order, CATEGORY_GP, "TIME_UNHANDCUFFING",
                "Time to unhandcuffing someone, in seconds. [default: 10]", "10").getInt();

        TIME_TO_GO_IN_JAIL = (byte)addVariableConfig(order, CATEGORY_GP, "TIME_TO_GO_IN_JAIL",
                "Time to go in jail when someone is handcuffed. [default: 20]", "20").getInt();

        FORMAT_TIME_JAIL = addVariableConfig(order, CATEGORY_GP, "FORMAT_TIME_JAIL",
                "Format of the time print when a player going to jail; 1=dd/MM/yyyy - HH:mm:ss; 2=dd/MM - HH:mm:ss; 3=dd - HH:mm:ss; 4=HH:mm:ss. [default: 4]", "4").getInt();

        DEFAULT_PLACE_CELL = addVariableConfig(order, CATEGORY_GP, "DEFAULT_PLACE_CELL",
                "Number of place in each cell by default. [default: 2]", "2").getInt();

        OP_JAIL_MANAGER = addVariableConfig(order, CATEGORY_GP, "OP_JAIL_MANAGER",
                "Only op can use jail manager to create and delete cell. [default: true]", "true").getBoolean();

        MULT_JAIL_TIME = addVariableConfig(order, CATEGORY_GP, "MULT_JAIL_TIME",
                "Multiplier time to go in jail. [default: 7.2]", "7.2").getDouble();

        REDUCE_ALIGN_KILL = addVariableConfig(order, CATEGORY_GP, "REDUCE_ALIGN_KILL",
                "Soustration of the ratio when a player kill one other. [default: 33]", "33").getInt();

        NICKNAME_FOR_CRIMINAL = addVariableConfig(order, CATEGORY_GP, "NICKNAME_FOR_CRIMINAL",
                "Criminal tracker print the nickname or the minecraft name. [default: false]", "false").getBoolean();

        MAX_RATE_CRIMINAL = addVariableConfig(order, CATEGORY_GP, "MAX_RATE_CRIMINAL",
                "Max rate a criminal can reach. [default: 24000]", "24000").getInt();

        MIN_RATE_TO_HAVE_POSITION = addVariableConfig(order, CATEGORY_GP, "MIN_RATE_TO_HAVE_POSITION",
                "Min rate to display position for galactic patrol. [default: 1000]", "1000").getInt();

        MAX_TIME_BETWEEN_EACH_POSITION = addVariableConfig(order, CATEGORY_GP, "MAX_TIME_BETWEEN_EACH_POSITION",
                "Max time between each position give to galactic patrol in seconds. [default: 7200]", "7200").getInt();
        MAX_TIME_BETWEEN_EACH_POSITION *= 1000L;

        MIN_TIME_BETWEEN_EACH_POSITION = addVariableConfig(order, CATEGORY_GP, "MIN_TIME_BETWEEN_EACH_POSITION",
                "Min time between each position give to galactic patrol in seconds. [default: 60]", "60").getInt();
        MIN_TIME_BETWEEN_EACH_POSITION *= 1000L;

        REINCARNATION_RESET_CRIMINAL = addVariableConfig(order, CATEGORY_GP, "REINCARNATION_RESET_CRIMINAL",
                "Reincarnation reset criminal rate. [default: true]", "true").getBoolean();

        REINCARNATION_RESET_GP = addVariableConfig(order, CATEGORY_GP, "REINCARNATION_RESET_GP",
                "Remove the player from galactic patrol on reincarnation. [default: true]", "true").getBoolean();

        CRIMINAL_QUIT_IN_FIGHT_GO_TO_JAIL = addVariableConfig(order, CATEGORY_GP, "CRIMINAL_QUIT_IN_FIGHT_GO_TO_JAIL",
                "Criminal go to jail if quit during a fight with a patroller. [default: true]", "true").getBoolean();

        CRIMINAL_TIME_TO_QUIT_IN_FIGHT = addVariableConfig(order, CATEGORY_GP, "CRIMINAL_TIME_TO_QUIT_IN_FIGHT",
                "Time before a criminal can quit in fight with a patroller in seconds. [default: 20]", "20").getInt();
        CRIMINAL_TIME_TO_QUIT_IN_FIGHT *= 1000L;


        config.setCategoryPropertyOrder(CATEGORY_GP, order);


        if (config.hasChanged())
            config.save();
    }
    
    private static Property addVariableConfig(List<String> order, String category, String key, String comment, String defaultValue) {
        Property prop;
        prop = config.get(category, key, defaultValue);
        prop.comment = comment;
        order.add(prop.getName());
        return prop;
    }

    @SubscribeEvent
    public void onConfigurationChangeEvent(ConfigChangedEvent.OnConfigChangedEvent event) {
        if (event.modID.equalsIgnoreCase(Main.MODID))
            loadConfiguration();
    }

    public static void setExitJail() {
        if (pos_exit_jail != null)
            return;

        try {
            MinecraftServer server = MinecraftServer.getServer();
            WorldServer world = server.worldServerForDimension(0);
            ChunkCoordinates spawnPoint = world.getSpawnPoint();
            pos_exit_jail = new Pos(spawnPoint.posX, spawnPoint.posY, spawnPoint.posZ, 0);
        }
        catch (Exception e) {
            System.out.println(e.getMessage());
        }
    }

    public static Configuration getConfiguration() {return config;}

}
