package xamora.gp_dbc.system;

import xamora.gp_dbc.utils.Pos;
import net.minecraft.entity.player.EntityPlayer;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

public class Cell {

    private static int last_id = 0;
    private int id;
    private Pos pos;
    private int place;
    private List<Inmate> inmates;

    public Cell(int x, int y, int z) {
        this(new Pos(x, y, z), 1);
    }

    public Cell(int x, int y, int z, int dimension) {
        this(new Pos(x, y, z, dimension), 1);
    }

    public Cell(int x, int y, int z, int dimension, int place) {
        this(new Pos(x, y, z, dimension), place);
    }

    public Cell(int x, int y, int z, int dimension, int place, int id) {
        this(new Pos(x, y, z, dimension), place, id);
    }

    public Cell(Pos pos, int place) {
        this.pos = pos;
        this.place = place;
        this.inmates = new ArrayList<>();
        this.id = last_id++;
    }

    public Cell(Pos pos, int place, int id) {
        this.pos = pos;
        this.place = place;
        this.inmates = new ArrayList<>();
        this.id = id;
        last_id = Math.max(last_id, id) + 1;
    }

    public Inmate addInmate(EntityPlayer player, long sec) {
        return addInmate(new Inmate(this, player.getUniqueID(), sec));
    }

    public Inmate addInmate(Inmate inmate) {
        inmates.add(inmate);
        return inmate;
    }

    public boolean removeInmate(EntityPlayer player, long sec) {
        return removeInmate(new Inmate(this, player.getUniqueID(), sec));
    }

    public boolean removeInmate(Inmate inmate) {
        return inmates.remove(inmate);
    }

    public boolean isFullCell() {
        return inmates.size() >= place;
    }

    public Pos getPos() { return pos; }
    public int getPlace() { return place; }
    public void setPlace(int place) { this.place = place; }
    public List<Inmate> getInmates() { return inmates; }

    public Inmate getInmate(UUID uuid) {
        for (Inmate inmate : inmates)
            if (inmate.getUuid().equals(uuid))
                return inmate;
        return null;
    }

    public int getId() { return id; }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;

        if (!(obj instanceof Cell))
            return false;

        Cell objCell = (Cell)obj;
        return pos.equals(objCell.getPos());
    }

    public boolean equals(int x, int y, int z, int dimension) {
        return pos.x == x && pos.y == y && pos.z == z && pos.dimension == dimension;
    }

    public boolean equals(int id) {
        return this.id == id;
    }

    @Override
    public String toString() {
        String inmates = "";
        for (Inmate inmate : this.inmates)
            inmates += inmate.toString() + "&";

        return pos.x + "@" + pos.y + "@" + pos.z + "@" + pos.dimension + "$" + place + "$" + id + "$" + inmates;
    }

}
