package xamora.gp_dbc.system;

import cpw.mods.fml.common.eventhandler.SubscribeEvent;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import xamora.gp_dbc.config.ConfigManager;
import xamora.gp_dbc.utils.Data;
import xamora.gp_dbc.utils.PlayerUtils;
import xamora.gp_dbc.utils.Pos;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.server.MinecraftServer;
import net.minecraftforge.event.entity.living.LivingDeathEvent;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

public class Criminals {

    public List<Criminal> criminals = new ArrayList<>();

    public Criminals() {
        try {
            criminals = listStringToCriminal(Data.loadDataString("criminals"));
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }
    }

    @SideOnly(Side.SERVER)
    @SubscribeEvent
    public void checkDeath(LivingDeathEvent event) {
        if (!(event.entity instanceof EntityPlayer))
            return;

        EntityPlayer dead = (EntityPlayer) event.entity;
        Entity entityKiller = event.entityLiving.func_94060_bK();
        if (!(entityKiller instanceof EntityPlayer) || entityKiller.getUniqueID().equals(dead.getUniqueID()))
            return;

        EntityPlayer killer = (EntityPlayer) entityKiller;

        Criminal criminal = null;
        if (isCriminal(killer.getUniqueID()))
            criminal = getCriminal(killer.getUniqueID());
        else
            criminal = addCriminal(killer, 0);

        criminal.addKill(dead);
        Data.saveDataString(listCriminalToString(criminals), "criminals");
    }

    public Criminal getCriminal(UUID uuid) {
        for (Criminal criminal : criminals)
            if (criminal.getUuid().equals(uuid))
                return criminal;
        return null;
    }

    public boolean isCriminal(UUID uuid) {
        return getCriminal(uuid) != null;
    }


    public Criminal addCriminal(EntityPlayer player, long rate) {

        Criminal criminal = null;
        if (isCriminal(player.getUniqueID())) {
            criminal = getCriminal(player.getUniqueID());
            criminal.setRate(criminal.getRate() + rate);
        }
        else {
            criminal = new Criminal(player, rate);
            criminals.add(criminal);
        }

        Data.saveDataString(listCriminalToString(criminals), "criminals");

        return criminal;
    }

    public boolean removeCriminal(UUID uuid_criminal) {
        for (Criminal criminal : criminals) {
            if (criminal.getUuid().equals(uuid_criminal)) {
                criminals.remove(criminal);
                Data.saveDataString(listCriminalToString(criminals), "criminals");
                return true;
            }
        }
        return false;
    }

    @SideOnly(Side.SERVER)
    public Pos getUpdatePos(UUID uuid) {
        if (!isCriminal(uuid))
            return null;
        Criminal criminal = getCriminal(uuid);

        EntityPlayer player = PlayerUtils.getPlayerFromUUID(uuid);
        if (player == null)
            return criminal.getLastPos();

        long rate = criminal.getRate();
        long next_time_pos = (long) (ConfigManager.MAX_TIME_BETWEEN_EACH_POSITION - ((double)rate / ConfigManager.MAX_RATE_CRIMINAL) * (ConfigManager.MAX_TIME_BETWEEN_EACH_POSITION - ConfigManager.MIN_TIME_BETWEEN_EACH_POSITION));

        long last_time = criminal.getLastPosTime();
        if (System.currentTimeMillis() - last_time <= next_time_pos)
            return criminal.getLastPos();

        String worldName = MinecraftServer.getServer().worldServerForDimension(player.dimension).provider.getDimensionName();
        Pos pos = new Pos((int) player.posX, (int) player.posY, (int) player.posZ, player.dimension, worldName);
        criminal.setPos(pos);

        Data.saveDataString(listCriminalToString(criminals), "criminals");
        return pos;
    }

    public static List<String> listCriminalToString(List<Criminal> criminals) {
        List<String> criminals_string = new ArrayList<>();

        for (Criminal criminal : criminals)
            criminals_string.add(criminal.toString());
        return criminals_string;
    }

    public static List<Criminal> listStringToCriminal(List<String> criminals_string) {
        List<Criminal> criminals = new ArrayList<>();

        for (String criminal : criminals_string) {
            String[] split = criminal.split("\\$");

            try {
                Pos pos = null;
                if (split.length > 4) {
                    String[] split_pos = split[4].split("@");
                    pos = new Pos(Integer.parseInt(split_pos[0]), Integer.parseInt(split_pos[1]), Integer.parseInt(split_pos[2]), Integer.parseInt(split_pos[3]), split_pos[4]);
                }

                criminals.add(new Criminal(UUID.fromString(split[0]), split[1], Integer.parseInt(split[2]), Long.parseLong(split[3]), pos));
            } catch (Exception e) {
                System.out.println(e);
            }
        }
        return criminals;
    }

}
