package xamora.gp_dbc.system;

import JinRyuu.JRMCore.server.JGPlayerMP;
import cpw.mods.fml.common.eventhandler.SubscribeEvent;
import cpw.mods.fml.common.gameevent.PlayerEvent;
import cpw.mods.fml.common.gameevent.TickEvent;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import xamora.gp_dbc.config.ConfigManager;
import xamora.gp_dbc.utils.Data;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.ChatComponentText;
import net.minecraft.util.EnumChatFormatting;
import net.minecraftforge.event.entity.living.LivingHurtEvent;

import java.util.HashMap;
import java.util.List;
import java.util.UUID;

import static net.minecraft.util.StatCollector.translateToLocal;

public class GalacticPatrol {

    private List<UUID> patrollers;
    public Jail jail;
    public Criminals criminals;

    public HashMap<UUID, Long> InFightWithPatrollers = new HashMap<>();

    public GalacticPatrol() {
        patrollers = Data.loadDataUUID("galactic_patrols");
        jail = new Jail();
        criminals = new Criminals();
    }

    public int addToGalaticPatrol(EntityPlayer player) {
        return addToGalaticPatrol(player, false);
    }

    public int addToGalaticPatrol(EntityPlayer player, boolean chatError) {
        return addToGalaticPatrol((EntityPlayerMP)player, chatError);
    }

    public int addToGalaticPatrol(EntityPlayerMP player) {
        return addToGalaticPatrol(player, false);
    }

    public int addToGalaticPatrol(EntityPlayerMP player, boolean chatError) {

        JGPlayerMP jgPlayer = new JGPlayerMP(player);
        jgPlayer.connectBaseNBT();
        int align = jgPlayer.getAlignment();


        if (align < ConfigManager.MIN_ALIGN_TO_JOIN) {
            if (chatError)
                player.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + translateToLocal("error.min_align_to_join")));
            return 2;
        }
        else if (align > ConfigManager.MAX_ALIGN_TO_JOIN) {
            if (chatError)
                player.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + translateToLocal("error.max_align_to_join")));
            return 3;
        }

        if (!patrollers.contains(player.getUniqueID())) {
            patrollers.add(player.getUniqueID());
            player.addChatMessage(new ChatComponentText(EnumChatFormatting.GREEN + translateToLocal("success.join_galactic_patrol")));
        }
        else {
            player.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + translateToLocal("error.join_galactic_patrol")));
            return 4;
        }

        Data.saveDataUUID(patrollers, "galactic_patrols");

        return 0;
    }

    private void addForceToGalacticPatrol(UUID uuid) {
        patrollers.add(uuid);
        Data.saveDataUUID(patrollers, "galactic_patrols");
    }

    public int removeFromGalaticPatrol(EntityPlayer player) {
        return removeFromGalaticPatrol(player, false);
    }

    public int removeFromGalaticPatrol(EntityPlayer player, boolean chatError) {
        return removeFromGalaticPatrol((EntityPlayerMP)player, chatError);
    }

    public int removeFromGalaticPatrol(EntityPlayerMP player) {
        return removeFromGalaticPatrol(player, false);
    }

    public int removeFromGalaticPatrol(EntityPlayerMP player, boolean chatError) {

        boolean removed = patrollers.remove(player.getUniqueID());
        Data.saveDataUUID(patrollers, "galactic_patrols");

        if (removed)
            player.addChatMessage(new ChatComponentText(EnumChatFormatting.GREEN + translateToLocal("success.leave_galactic_patrol")));
        else
            player.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + translateToLocal("error.leave_galactic_patrol")));

        return removed ? 0 : 1;
    }

    public boolean isInGalacticPatrol(EntityPlayer player) {
        return patrollers.contains(player.getUniqueID());
    }

    public boolean isInGalacticPatrol(EntityPlayerMP player) {
        return patrollers.contains(player.getUniqueID());
    }

    public boolean isInGalacticPatrol(UUID uuid) {
        return patrollers.contains(uuid);
    }

    public List<UUID> getPatrollers() { return patrollers; }

    @SubscribeEvent
    public void checkReincarnated(TickEvent.ServerTickEvent event) {
        List<EntityPlayer> playerList = MinecraftServer.getServer().getConfigurationManager().playerEntityList;

        for (EntityPlayer player : playerList) {
            JGPlayerMP jgPlayer = new JGPlayerMP(player);
            NBTTagCompound nbt = jgPlayer.connectBaseNBT();
            boolean isReincarnated = nbt.getInteger("jrmcRencrnt") == 1;

            if (isReincarnated) {
                if (ConfigManager.REINCARNATION_RESET_GP && isInGalacticPatrol(player))
                    removeFromGalaticPatrol(player);
                if (ConfigManager.REINCARNATION_RESET_CRIMINAL && criminals.isCriminal(player.getUniqueID()))
                    criminals.removeCriminal(player.getUniqueID());
            }
        }
    }

    @SubscribeEvent
    @SideOnly(Side.SERVER)
    public void checkFight(LivingHurtEvent event) {
        if (!ConfigManager.CRIMINAL_QUIT_IN_FIGHT_GO_TO_JAIL)
            return;

        if (!(event.source.getSourceOfDamage() instanceof EntityPlayer) || !(event.entity instanceof EntityPlayer))
            return;

        EntityPlayer source = (EntityPlayer) event.source.getSourceOfDamage();
        if (!isInGalacticPatrol(source))
            return;

        EntityPlayer target = (EntityPlayer) event.entity;
        if (isInGalacticPatrol(target))
            return;

        InFightWithPatrollers.put(source.getUniqueID(), System.currentTimeMillis());
    }

    @SubscribeEvent
    @SideOnly(Side.SERVER)
    public void checkFightLogOut(PlayerEvent.PlayerLoggedOutEvent event) {
        if (ConfigManager.CRIMINAL_QUIT_IN_FIGHT_GO_TO_JAIL)
            return;

        Long time = InFightWithPatrollers.get(event.player.getUniqueID());
        if (time == null || System.currentTimeMillis() - time <= ConfigManager.CRIMINAL_TIME_TO_QUIT_IN_FIGHT)
            return;


    }

}
